<?php

namespace Drupal\entity_mesh\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_mesh\ProcessDataForD3Trait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for Entity Mesh page route.
 */
class EntityMeshController extends ControllerBase {

  use ProcessDataForD3Trait;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  final public function __construct(Connection $database, EntityTypeManagerInterface $entityTypeManager) {
    $this->database = $database;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  final public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Entity Mesh Items Query.
   */
  public function query() {

    $langcode = $this->languageManager()->getCurrentLanguage()->getId();

    $query = $this->database->select('node_field_data', 'snfd');

    $query->leftJoin('entity_mesh', 'em', 'em.source_entity_id = snfd.nid AND em.source_entity_type = :node AND em.source_entity_langcode = snfd.langcode',
      [':node' => 'node']);
    $query->leftJoin('node_field_data', 'tnfd', 'em.target_entity_id = tnfd.nid AND em.target_entity_type = :node AND em.source_entity_langcode = tnfd.langcode',
      [':node' => 'node']);

    $query->condition('snfd.langcode', $langcode);

    $query->fields('em');
    $query->addField('snfd', 'title', 'source_title');
    $query->addField('snfd', 'type', 'source_type');
    $query->addField('tnfd', 'title', 'target_title');
    $query->addField('tnfd', 'type', 'target_type');

    return $query;
  }

  /**
   * Load Query results.
   */
  public function results() {
    $query = $this->query();
    return $query->execute()->fetchAll();
  }

  /**
   * Builds the response.
   */
  public function build() {

    $build = [];

    // @todo Dependency injection!
    $results = $this->results();

    $build['graph'] = [
      '#type' => 'item',
      '#markup' => '<div id="entity-mesh-graph"></div>',
    ];

    if (is_array($results) && count($results) > 0) {
      $build['#attached']['drupalSettings']['entity_mesh']['data'] = $this->processData($results);
      $build['#attached']['drupalSettings']['entity_mesh']['settings'] = ['fullHeight' => FALSE];
      $build['#attached']['library'][] = 'entity_mesh/entity_mesh';
      $build['#attached']['library'][] = 'entity_mesh/d3';
    }
    return $build;
  }

}
