<?php

namespace Drupal\entity_mesh;

/**
 * Service to perform database operations.
 */
interface RepositoryInterface {

  /**
   * The entity mesh table.
   */
  const ENTITY_MESH_TABLE = 'entity_mesh';

  /**
   * Get the database service.
   */
  public function getDatabaseService();

  /**
   * Get logger.
   */
  public function getlogger();

  /**
   * Insert multiple rows in one execution.
   *
   * @param \Drupal\entity_mesh\SourceInterface $source
   *   Source target object.
   *
   * @return bool
   *   TRUE if the operation was successful, FALSE otherwise.
   */
  public function insertSource(SourceInterface $source): bool;

  /**
   * Remove all the rows of one source.
   *
   * @param \Drupal\entity_mesh\SourceInterface $source
   *   Source object.
   *
   * @return bool
   *   TRUE if the operation was successful, FALSE otherwise.
   */
  public function deleteSource(SourceInterface $source): bool;

  /**
   * Remove all the rows of one source by type.
   *
   * @param string $type
   *   The type of the source.
   * @param array $conditions
   *   Additional conditions to delete the source.
   *
   * @return bool
   *   TRUE if the operation was successful, FALSE otherwise.
   */
  public function deleteSourceByType(string $type, array $conditions): bool;

  /**
   * Remove all the rows.
   *
   * @return bool
   *   TRUE if the operation was successful, FALSE otherwise.
   */
  public function deleteAllSourceTarget(): bool;

  /**
   * Save the source and its targets in database.
   *
   * @param \Drupal\entity_mesh\SourceInterface $source
   *   Source with targets to save.
   *
   * @return bool
   *   TRUE if the operation was successful, FALSE otherwise.
   */
  public function saveSource(SourceInterface $source): bool;

  /**
   * Instance an empty source object.
   *
   * @return \Drupal\entity_mesh\SourceInterface
   *   The source object.
   */
  public function instanceEmptySource(): SourceInterface;

  /**
   * Instance an empty target object.
   *
   * @return \Drupal\entity_mesh\TargetInterface
   *   The target object.
   */
  public function instanceEmptyTarget(): TargetInterface;

}
